﻿using Phidget22;
using Phidget22.ExampleUtils;
using System;
using System.Linq;
using System.Windows.Forms;

namespace GraphicLCD_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		LCD openCh = null;
		LCD lcd = null;
		private ErrorEventBox errorBox;
		int LCD_WIDTH_PIXELS = 128;
		int LCD_HEIGHT_PIXELS = 64;

		int sleepCNT;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
		}

		public Form1(LCD ch) {
			InitializeComponent();
			openCh = ch;
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			lcd = openCh ?? openArgs.makeChannel<LCD>();

			this.Size = this.MinimumSize;

			lcd.Attach += lcd_attach;
			lcd.Detach += lcd_detach;
			lcd.Error += lcd_error;

			try {
				lcd.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			timer2.Stop();

			lcd.Attach -= lcd_attach;
			lcd.Detach -= lcd_detach;
			lcd.Error -= lcd_error;

			lcd.Close();
		}

		void lcd_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox2.FillPhidgetInfo((Phidget)sender);

			try {
				lcd.Sleeping = false;
				//these are just generic defaults, they can be changed to suit your application
				lcd.Backlight = 0.5; //50% backlight
				contrasttrk.Value = (int)(lcd.Contrast * 255.0);
				brightnesstrk.Value = (int)(lcd.Backlight * 255.0);

				//load whatever is stored in flash memory
				lcd.FrameBuffer = 2;
				lcd.Flush();

				LCD_WIDTH_PIXELS = lcd.Width;
				LCD_HEIGHT_PIXELS = lcd.Height;
			} catch (PhidgetException ex) { errorBox.addMessage("Error initializing device: " + ex.Message); }

			settingsBox.Visible = true;
		}


		void lcd_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			timer2.Stop();
			phidgetInfoBox2.Clear();
			settingsBox.Visible = false;
		}


		void lcd_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		private void contrast_Scroll(object sender, EventArgs e) {
			try {
				lcd.Contrast = ((double)contrasttrk.Value) / 255.0;
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting contrast: " + ex.Message); }
		}

		private void brightnesstrk_Scroll(object sender, EventArgs e) {
			try {
				lcd.Backlight = ((double)brightnesstrk.Value) / 255.0;
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting brightness: " + ex.Message); }
		}

		public void DrawLineAsyncCallback(IAsyncResult result) {
			try {
				lcd.EndDrawLine(result);
			} catch (PhidgetException ex) { errorBox.addMessage("Async Error drawing line: " + ex.Message); }
		}
		public void DrawRectAsyncCallback(IAsyncResult result) {
			try {
				lcd.EndDrawRectangle(result);
			} catch (PhidgetException ex) { errorBox.addMessage("Async Error drawing rect: " + ex.Message); }
		}
		public void CopyAsyncCallback(IAsyncResult result) {
			try {
				lcd.EndCopy(result);
			} catch (PhidgetException ex) { errorBox.addMessage("Async Error copying: " + ex.Message); }
		}

		private void rectSamplebtn_Click(object sender, EventArgs e) {
			timer2.Stop();
			try {
				//select frame buffer to draw to
				lcd.FrameBuffer = 0;
				lcd.Clear();

				//draw some stuff asynchronously
				for (int i = 0; i < 8; i++)
					lcd.BeginDrawLine(0, (i * 8), (LCD_WIDTH_PIXELS - 1), (i * 8), new AsyncCallback(DrawLineAsyncCallback), null);

				for (int i = 0; i < 21; i++)
					lcd.BeginDrawRectangle((i * 6), 0, ((i * 6) + 6), 7, true, (i % 2) == 0 ? false : true, new AsyncCallback(DrawRectAsyncCallback), null);

				lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 9, false, new AsyncCallback(CopyAsyncCallback), null);
				lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 17, false, new AsyncCallback(CopyAsyncCallback), null);
				lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 25, false, new AsyncCallback(CopyAsyncCallback), null);
				lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1), 6, 0, 33, false, new AsyncCallback(CopyAsyncCallback), null);
				lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 41, false, new AsyncCallback(CopyAsyncCallback), null);
				lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 49, false, new AsyncCallback(CopyAsyncCallback), null);
				lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 57, false, new AsyncCallback(CopyAsyncCallback), null);

				//write the frame buffer to the screen
				lcd.Flush();
			} catch (PhidgetException ex) { errorBox.addMessage("Error drawing image: " + ex.Message); }
		}

		private void basicExamplebtn_Click(object sender, EventArgs e) {
			timer2.Stop();
			byte[] aBitmap;
			byte[] csBitmap;

			try {
				//select the frame buffer to draw to
				lcd.FrameBuffer = 0;
				lcd.Clear();

				#region bitmapDefinitions
				//NOTE: I'm using 2 character per bit strings to increase readability in the code. Actual functions require byte arrays.

				aBitmap = getBitmapBytes(
					  "      ██████      "
					+ "    ██████████    "
					+ "  ████      ████  "
					+ "  ████      ████  "
					+ "  ██████████████  "
					+ "  ██████████████  "
					+ "  ████      ████  "
					+ "  ████      ████  "
					+ "  ████      ████  "
					+ "  ████      ████  ");

				csBitmap = getBitmapBytes(
					  "          ██████              ████████                                                        "
					+ "        ██      ██          ██        ██                                                      "
					+ "      ██          ██      ██            ██                                                    "
					+ "    ██            ██    ██              ██                                                    "
					+ "  ██              ██  ██                ██                                                    "
					+ "  ██              ██  ██              ██                                                      "
					+ "██                ██  ██            ██                                                        "
					+ "██              ██    ██                                                                      "
					+ "██            ██      ██                        ██                                            "
					+ "██        ████          ██                      ██                                            "
					+ "██                        ██                    ██                                            "
					+ "██                          ██                ██                                              "
					+ "██                ██          ████            ██                                              "
					+ "██              ██                ████        ██                                              "
					+ "██              ██      ██            ██      ██████        ████      ██      ████  ████████  "
					+ "██            ██      ██                ██  ██      ██    ██    ██      ██  ██      ██      ██"
					+ "██          ██        ██                ██  ██      ██  ██      ██      ████        ██      ██"
					+ "  ██      ██          ██              ██    ██      ██  ██      ██      ██          ██      ██"
					+ "    ██████              ██          ██      ██      ██  ██    ████    ██            ██    ██  "
					+ "                          ██████████      ██        ████  ████    ██  ██            ██████    "
					+ "                                                                                    ██        "
					+ "                                                                                    ██        "
					+ "                                                                                  ██          "
					+ "                                                                                  ██          "
					+ "                                                                                  ██          ");

				#endregion

				//draw some stuff
				lcd.WriteText(LCDFont.Dimensions_6x12, 0, 0, "A few boldface 'A's");
				lcd.SetFontSize(LCDFont.User1, 9, 10);
				lcd.SetCharacterBitmap(LCDFont.User1, "A", aBitmap);
				lcd.WriteText(LCDFont.User1, 0, 13, "AAAAAA");
				lcd.DrawLine(5, 40, 35, 30);
				lcd.DrawRectangle(20, 35, 30, 45, true, false);
				lcd.Copy(0, 0, 5, 30, 35, 45, 87, 30, false);
				lcd.WriteBitmap(40, 35, 47, 25, csBitmap);
				lcd.DrawPixel(64, 32, LCDPixelState.On);       //this pixel is in the centre of your screen

				//write the frame buffer to the screen
				lcd.Flush();
			} catch (PhidgetException ex) { errorBox.addMessage("Error drawing image: " + ex.Message); }
		}

		private void saveFrameBtn_Click(object sender, EventArgs e) {
			timer2.Stop();
			try {
				//copy the contents of frame buffer 0 to frame buffer 2
				lcd.Copy(0, 2, 0, 0, LCD_WIDTH_PIXELS - 1, LCD_HEIGHT_PIXELS - 1, 0, 0, false);

				//store frame buffer 2 in flash memory so it persists through power cycling.
				lcd.SaveFrameBuffer(2);
			} catch (PhidgetException ex) { errorBox.addMessage("Error saving: " + ex.Message); }
		}

		private void sleepbtn_Click(object sender, EventArgs e) {
			timer2.Stop();
			try {
				//put the device to sleep.  useful for preserving backlight life and power saving
				sleepCNT++;
				lcd.Sleeping = (sleepCNT % 2) == 0 ? false : true;
			} catch (PhidgetException ex) { errorBox.addMessage("Error sleeping: " + ex.Message); }
		}

		private void recallBtn_Click(object sender, EventArgs e) {
			timer2.Stop();
			try {
				//write stored data from frame buffer 2 to the screen
				lcd.FrameBuffer = 2;
				lcd.Flush();
			} catch (PhidgetException ex) { errorBox.addMessage("Error recalling frame: " + ex.Message); }
		}


		private void clearBtn_Click(object sender, EventArgs e) {
			timer2.Stop();
			try {
				//clear frame buffer 0 and write it to the screen
				lcd.FrameBuffer = 0;
				lcd.Clear();
				lcd.Flush();
			} catch (PhidgetException ex) { errorBox.addMessage("Error clearing screen: " + ex.Message); }
		}

		private byte[] getBitmapBytes(String charStr) {
			Byte[] bytes = new byte[charStr.Length / 2];
			for (int i = 0; i < charStr.Length / 2; i++)
				bytes[i] = (byte)(charStr.ToArray()[i * 2] == ' ' ? 0x00 : 0x01);
			return bytes;
		}
	}
}
