﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Threading;
using Phidget22;
using Phidget22.ExampleUtils;
using System.Diagnostics;

namespace GraphicLCD_Example
{
    public partial class Form1 : Form
    {
        CommandLineOpen open;
        LCD lcd = null;
        private ErrorEventBox errorBox;
        int LCD_WIDTH_PIXELS = 128;
        int LCD_HEIGHT_PIXELS = 64;

        int sleepCNT;

        public Form1()
        {
            open = new CommandLineOpen(this);
            InitializeComponent();
        }

        public Form1(String[] commandLine)
        {
            open = new CommandLineOpen(this);
            open.commandLine = commandLine;
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Phidget.InvokeEventCallbacks = true;
            errorBox = new ErrorEventBox(this);
            lcd = new LCD();

            this.Size = this.MinimumSize;

            lcd.Attach += lcd_attach;
            lcd.Detach += lcd_detach;
            lcd.Error += lcd_error;

            commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

            try
            { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
                lcd.Channel = phidgetParameters.Channel; //selects the channel on the device to open
                lcd.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
                lcd.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
                lcd.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

                if (phidgetParameters.isRemote) //are we trying to open a remote device?
                {
                    lcd.IsRemote = true;
                    Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
                    if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
                        Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
                }
                else
                    lcd.IsLocal = true;

                lcd.Open(); //open the device specified by the above parameters
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (writeAnimatedGifThread != null && writeAnimatedGifThread.IsAlive)
            {
                stop_writeAnimatedGifThread = true;
                writeAnimatedGifThread.Join();
            }
            if (writeScreenThread != null && writeScreenThread.IsAlive)
            {
                stop_writeScreenThread = true;
                writeScreenThread.Join();
            }

            if (asyncTextThread != null)
            {
                asyncTextThreadRun = false;
                asyncTextThread.Join();
                asyncTextThread = null;
            }

            timer2.Stop();

            //lcd.Close();
        }

        void lcd_attach(object sender, Phidget22.Events.AttachEventArgs e)
        {
            phidgetInfoBox2.FillPhidgetInfo((Phidget)sender);

            try
            {
                //lcd.Sleeping = false;
                //these are just generic defaults, they can be changed to suit your application
                lcd.Backlight = 0.5; //50% backlight
                lcd.Contrast = 0.25; //25% contrast
                contrasttrk.Value = (int)(lcd.Contrast * 255.0);
                brightnesstrk.Value = (int)(lcd.Backlight * 255.0);

                //load whatever is stored in flash memory
                lcd.FrameBuffer = 2;
                //lcd.Flush();

                LCD_WIDTH_PIXELS = lcd.Width;
                LCD_HEIGHT_PIXELS = lcd.Height;

                lcd.HubPortSpeed = 1000000;
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error initializing device: " + ex.Message); }

            settingsBox.Visible = true;
        }


        void lcd_detach(object sender, Phidget22.Events.DetachEventArgs e)
        {
            timer2.Stop();
            phidgetInfoBox2.Clear();
            settingsBox.Visible = false;
        }


        void lcd_error(object sender, Phidget22.Events.ErrorEventArgs e)
        {
            errorBox.addMessage(e.Description);
        }

        private void contrast_Scroll(object sender, EventArgs e)
        {
            try
            {
                lcd.Contrast = ((double)contrasttrk.Value) / 255.0;
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error setting contrast: " + ex.Message); }
        }

        private void brightnesstrk_Scroll(object sender, EventArgs e)
        {
            try
            {
                lcd.Backlight = ((double)brightnesstrk.Value) / 255.0;
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error setting brightness: " + ex.Message); }
        }

        public void DrawLineAsyncCallback(IAsyncResult result)
        {
            try
            {
                lcd.EndDrawLine(result);
            }
            catch (PhidgetException ex) { errorBox.addMessage("Async Error drawing line: " + ex.Message); }
        }
        public void DrawRectAsyncCallback(IAsyncResult result)
        {
            try
            {
                //lcd.EndDrawRectangle(result);
            }
            catch (PhidgetException ex) { errorBox.addMessage("Async Error drawing rect: " + ex.Message); }
        }
        public void CopyAsyncCallback(IAsyncResult result)
        {
            try
            {
                lcd.EndCopy(result);
            }
            catch (PhidgetException ex) { errorBox.addMessage("Async Error copying: " + ex.Message); }
        }

        private void rectSamplebtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            try
            {
                //select frame buffer to draw to
                lcd.FrameBuffer = 0;
                //lcd.Clear();

                //draw some stuff asynchronously
                for (int i = 0; i < 8; i++)
                    lcd.BeginDrawLine(0, (i * 8), (LCD_WIDTH_PIXELS - 1), (i * 8), new AsyncCallback(DrawLineAsyncCallback), null);

                for (int i = 0; i < 21; i++)
                    //lcd.BeginDrawRectangle((i * 6), 0, ((i * 6) + 6), 7, true, (i % 2) == 0 ? false : true, new AsyncCallback(DrawRectAsyncCallback), null);

                    lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 9, false, new AsyncCallback(CopyAsyncCallback), null);
                lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 17, false, new AsyncCallback(CopyAsyncCallback), null);
                lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 25, false, new AsyncCallback(CopyAsyncCallback), null);
                lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1), 6, 0, 33, false, new AsyncCallback(CopyAsyncCallback), null);
                lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 41, false, new AsyncCallback(CopyAsyncCallback), null);
                lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 49, false, new AsyncCallback(CopyAsyncCallback), null);
                lcd.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 57, false, new AsyncCallback(CopyAsyncCallback), null);

                //write the frame buffer to the screen
                //lcd.Flush();
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error drawing image: " + ex.Message); }
        }

        private void basicExamplebtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            byte[] aBitmap;
            byte[] csBitmap;

            try
            {
                //select the frame buffer to draw to
                lcd.FrameBuffer = 0;
                //lcd.Clear();

                #region bitmapDefinitions
                //NOTE: I'm using 2 character per bit strings to increase readability in the code. Actual functions require byte arrays.

                aBitmap = getBitmapBytes(
                      "      ██████      "
                    + "    ██████████    "
                    + "  ████      ████  "
                    + "  ████      ████  "
                    + "  ██████████████  "
                    + "  ██████████████  "
                    + "  ████      ████  "
                    + "  ████      ████  "
                    + "  ████      ████  "
                    + "  ████      ████  ");

                csBitmap = getBitmapBytes(
                      "          ██████              ████████                                                        "
                    + "        ██      ██          ██        ██                                                      "
                    + "      ██          ██      ██            ██                                                    "
                    + "    ██            ██    ██              ██                                                    "
                    + "  ██              ██  ██                ██                                                    "
                    + "  ██              ██  ██              ██                                                      "
                    + "██                ██  ██            ██                                                        "
                    + "██              ██    ██                                                                      "
                    + "██            ██      ██                        ██                                            "
                    + "██        ████          ██                      ██                                            "
                    + "██                        ██                    ██                                            "
                    + "██                          ██                ██                                              "
                    + "██                ██          ████            ██                                              "
                    + "██              ██                ████        ██                                              "
                    + "██              ██      ██            ██      ██████        ████      ██      ████  ████████  "
                    + "██            ██      ██                ██  ██      ██    ██    ██      ██  ██      ██      ██"
                    + "██          ██        ██                ██  ██      ██  ██      ██      ████        ██      ██"
                    + "  ██      ██          ██              ██    ██      ██  ██      ██      ██          ██      ██"
                    + "    ██████              ██          ██      ██      ██  ██    ████    ██            ██    ██  "
                    + "                          ██████████      ██        ████  ████    ██  ██            ██████    "
                    + "                                                                                    ██        "
                    + "                                                                                    ██        "
                    + "                                                                                  ██          "
                    + "                                                                                  ██          "
                    + "                                                                                  ██          ");

                #endregion

                //draw some stuff
                /*lcd.WriteText(LCDFont.Dimensions_6x12, 0, 0, "A few boldface 'A's");
				lcd.SetFontSize(LCDFont.User1, 9, 10);
				lcd.SetCharacterBitmap(LCDFont.User1, "A", aBitmap);
				lcd.WriteText(LCDFont.User1, 0, 13, "AAAAAA");
				lcd.DrawLine(5, 40, 35, 30);
				lcd.DrawRectangle(20, 35, 30, 45, true, false);
				lcd.Copy(0, 0, 5, 30, 35, 45, 87, 30, false);
				lcd.WriteBitmap(40, 35, 47, 25, csBitmap);
				lcd.DrawPixel(64, 32, LCDPixelState.On);       //this pixel is in the centre of your screen

				//write the frame buffer to the screen
				lcd.Flush();*/
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error drawing image: " + ex.Message); }
        }

        private void saveFrameBtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            try
            {
                //copy the contents of frame buffer 0 to frame buffer 2
                //lcd.Copy(0, 2, 0, 0, LCD_WIDTH_PIXELS - 1, LCD_HEIGHT_PIXELS - 1, 0, 0, false);

                //store frame buffer 2 in flash memory so it persists through power cycling.
                //lcd.SaveFrameBuffer(2);
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error saving: " + ex.Message); }
        }

        private void sleepbtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            try
            {
                //put the device to sleep.  useful for preserving backlight life and power saving
                sleepCNT++;
                //lcd.Sleeping = (sleepCNT % 2) == 0 ? false : true;
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error sleeping: " + ex.Message); }
        }

        private void recallBtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            try
            {
                //write stored data from frame buffer 2 to the screen
                lcd.FrameBuffer = 2;
                //lcd.Flush();
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error recalling frame: " + ex.Message); }
        }


        private void clearBtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            try
            {
                //clear frame buffer 0 and write it to the screen
                lcd.FrameBuffer = 0;
                //lcd.Clear();
                //lcd.Flush();
            }
            catch (PhidgetException ex) { errorBox.addMessage("Error clearing screen: " + ex.Message); }
        }

        private void asyncTxtBtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            //select the frame buffer to draw to
            lcd.FrameBuffer = 0;
            //lcd.Clear();

            //More readable in the code then a byte array
            Byte[] funnyface = getBitmapBytes(
                  "██        ██"
                + "  ██    ██  "
                + "    ████    "
                + "            "
                + "  ██    ██  "
                + "            "
                + "    ████    "
                + "            "
                + "██        ██"
                + "██        ██"
                + "  ██    ██  "
                + "    ████    ");

            Byte[] space = getBitmapBytes(
                  "            "
                + "            "
                + "            "
                + "            "
                + "            "
                + "            "
                + "            "
                + "            "
                + "            "
                + "            "
                + "            "
                + "            ");

            try
            {
                /*lcd.SetFontSize(LCDFont.User2, 6, 12);
				lcd.SetCharacterBitmap(LCDFont.User2, "0", funnyface);
				lcd.SetCharacterBitmap(LCDFont.User2, " ", space);
				lcd.WriteText(LCDFont.User2, 0, 35, "0 0 0");

				Byte[] thing1 = new Byte[10] { 1, 0, 1, 0, 1, 0, 1, 0, 1, 0 };
				Byte[] thing2 = new Byte[10] { 0, 1, 0, 1, 0, 1, 0, 1, 0, 1 };
				Byte[] space2 = new Byte[10] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
				lcd.SetFontSize(LCDFont.User1, 1, 10);
				lcd.SetCharacterBitmap(LCDFont.User1, "1", thing1);
				lcd.SetCharacterBitmap(LCDFont.User1, "2", thing2);
				lcd.SetCharacterBitmap(LCDFont.User1, " ", space2);*/
            }
            catch (PhidgetException ex)
            {
                errorBox.addMessage("Error setting up customs chars: " + ex.Message);
            }

            try
            {
                /*IAsyncResult ret1 = lcd.BeginWriteText(LCDFont.User1, 0, 0, "121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 12121212", null, null);
				IAsyncResult ret2 = lcd.BeginWriteText(LCDFont.User1, 0, 10, " 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 1212121", null, null);
				IAsyncResult ret3 = lcd.BeginWriteText(LCDFont.User1, 0, 20, "121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 12121212", null, null);
                
				lcd.EndWriteText(ret1);
				lcd.EndWriteText(ret2);
				lcd.EndWriteText(ret3);*/
            }
            catch (PhidgetException ex)
            {
                errorBox.addMessage("Async Error writing text: " + ex.Message);
            }

            //lcd.BeginWriteText(LCDFont.Dimensions_6x12, 5, 5, "Overwriting!", null, null);

            //write the frame buffer to the screen
            //lcd.Flush();
        }

        Random rand = new Random();
        Thread asyncTextThread;
        Boolean asyncTextThreadRun;
        private void asyncTextThreadFunction()
        {
            while (asyncTextThreadRun)
            {
                int x = rand.Next(127);
                int y = rand.Next(63);

                //IAsyncResult res1 = lcd.BeginWriteText(LCDFont.Dimensions_5x8, x, y, rand.NextDouble().ToString("F2"), null, null);
                IAsyncResult res2 = lcd.BeginFlush(null, null);

                try
                {
                    //lcd.EndWriteText(res1);
                }
                catch (PhidgetException ex)
                {
                    errorBox.addMessage("Async Error writing text: " + ex.Message);
                }
                try
                {
                    lcd.EndFlush(res2);
                }
                catch (PhidgetException ex)
                {
                    errorBox.addMessage("Async Error flushing: " + ex.Message);
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (asyncTextThread != null)
            {
                asyncTextThreadRun = false;
                asyncTextThread.Join();
                asyncTextThread = null;
            }
            else
            {
                asyncTextThread = new Thread(new ThreadStart(asyncTextThreadFunction));
                asyncTextThreadRun = true;
                asyncTextThread.Start();
            }
        }

        private byte[] getBitmapBytes(String charStr)
        {
            Byte[] bytes = new byte[charStr.Length / 2];
            for (int i = 0; i < charStr.Length / 2; i++)
                bytes[i] = (byte)(charStr.ToArray()[i * 2] == ' ' ? 0x00 : 0x01);
            return bytes;
        }

        public void ClearAsyncCallback(IAsyncResult result)
        {
            try
            {
                lcd.EndClear(result);
            }
            catch (PhidgetException ex) { errorBox.addMessage("Async Error on Clear: " + ex.Message); }
        }
        public void WriteBitmapAsyncCallback(IAsyncResult result)
        {
            try
            {
                lcd.EndWriteBitmap(result);
            }
            catch (PhidgetException ex) { errorBox.addMessage("Async Error on WriteBitmap: " + ex.Message); }
        }
        public void FlushAsyncCallback(IAsyncResult result)
        {
            try
            {
                lcd.EndFlush(result);
            }
            catch (PhidgetException ex) { errorBox.addMessage("Async Error on Flush: " + ex.Message); }
        }
        private void writeImage(Bitmap pic, double whitepoint = 0.7)
        {
            //resize to fit within lcd width and height
            int width = pic.Width;
            int height = pic.Height;
            if (width > lcd.Width)
            {
                double factor = (double)lcd.Width / (double)width;
                width = (int)(width * factor);
                height = (int)(height * factor);
            }
            if (height > lcd.Height)
            {
                double factor = (double)lcd.Height / (double)height;
                width = (int)(width * factor);
                height = (int)(height * factor);
            }
            Bitmap pic2 = new Bitmap(pic, width, height);

            //convert to 1-bit and copy to byte array
            Byte[] imageForLCD = new Byte[width * height];
            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    Color c = pic2.GetPixel(x, y);
                    if (c.GetBrightness() >= whitepoint)
                        imageForLCD[x + y * width] = 0;
                    else
                        imageForLCD[x + y * width] = 1;
                }
            }

            pictureBox1.Image = pic2;

            lcd.BeginClear(new AsyncCallback(ClearAsyncCallback), null);
            lcd.BeginWriteBitmap(0, 0, width, height, imageForLCD, new AsyncCallback(WriteBitmapAsyncCallback), null);
            //lcd.BeginFlush(new AsyncCallback(FlushAsyncCallback), null);
            lcd.Flush();
        }

        private void writeImageGreyscale(Bitmap pic)
        {
            //resize to fit within lcd width and height
            int width = pic.Width;
            int height = pic.Height;
            if (width > lcd.Width)
            {
                double factor = (double)lcd.Width / (double)width;
                width = (int)(width * factor);
                height = (int)(height * factor);
            }
            if (height > lcd.Height)
            {
                double factor = (double)lcd.Height / (double)height;
                width = (int)(width * factor);
                height = (int)(height * factor);
            }
            Bitmap pic2 = new Bitmap(pic, width, height);

            //convert to 1-bit and copy to byte array
            Byte[] imageForLCD0 = new Byte[width * height];
            Byte[] imageForLCD1 = new Byte[width * height];
            Byte[] imageForLCD2 = new Byte[width * height];

            Bitmap picForView = new Bitmap(width, height);

            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    //All to black
                    imageForLCD0[x + y * width] = 1;
                    imageForLCD1[x + y * width] = 1;
                    imageForLCD2[x + y * width] = 1;

                    //set to white as brighter
                    Color c = pic2.GetPixel(x, y);
                    picForView.SetPixel(x, y, Color.Black);
                    if (c.GetBrightness() >= 0.2)
                    {
                        imageForLCD0[x + y * width] = 0;
                        picForView.SetPixel(x, y, Color.DarkGray);
                    }
                    if (c.GetBrightness() >= 0.5)
                    {
                        imageForLCD1[x + y * width] = 0;
                        picForView.SetPixel(x, y, Color.LightGray);
                    }
                    if (c.GetBrightness() >= 0.8)
                    {
                        imageForLCD2[x + y * width] = 0;
                        picForView.SetPixel(x, y, Color.White);
                    }
                }
            }

            pictureBox1.Image = picForView;

            lcd.FrameBuffer = 0;
            lcd.BeginClear(new AsyncCallback(ClearAsyncCallback), null);
            lcd.BeginWriteBitmap(0, 0, width, height, imageForLCD0, new AsyncCallback(WriteBitmapAsyncCallback), null);
            lcd.BeginFlush(new AsyncCallback(FlushAsyncCallback), null);

            lcd.FrameBuffer = 1;
            lcd.BeginClear(new AsyncCallback(ClearAsyncCallback), null);
            lcd.BeginWriteBitmap(0, 0, width, height, imageForLCD1, new AsyncCallback(WriteBitmapAsyncCallback), null);
            lcd.BeginFlush(new AsyncCallback(FlushAsyncCallback), null);

            lcd.FrameBuffer = 2;
            lcd.BeginClear(new AsyncCallback(ClearAsyncCallback), null);
            lcd.BeginWriteBitmap(0, 0, width, height, imageForLCD2, new AsyncCallback(WriteBitmapAsyncCallback), null);
            lcd.BeginFlush(new AsyncCallback(FlushAsyncCallback), null);

            timer2.Start();
        }

        private void timer2_Tick(object sender, EventArgs e)
        {
            if (lcd.FrameBuffer == 0)
                lcd.BeginSetFrameBuffer(1, null, null);
            else if (lcd.FrameBuffer == 1)
                lcd.BeginSetFrameBuffer(2, null, null);
            else
                lcd.BeginSetFrameBuffer(0, null, null);
            lcd.BeginFlush(null, null);
        }

        Image image;
        Thread writeAnimatedGifThread;
        Boolean stop_writeAnimatedGifThread = false;
        double fps = 0;
        void writeAnimatedGifThreadFunction()
        {
            int repeat = 2;
            int numberOfFrames = image.GetFrameCount(FrameDimension.Time);
            while (repeat-- > 0)
            {
                for (int i = 0; i < numberOfFrames; i++)
                {
                    Stopwatch sw = new Stopwatch();
                    sw.Start();

                    image.SelectActiveFrame(FrameDimension.Time, i);
                    PropertyItem item = image.GetPropertyItem(0x5100); // FrameDelay in libgdiplus
                                                                       // Time is in 1/100ths of a second; convert to ms
                    long delay = (item.Value[0] + item.Value[1] * 256) * 10;
                    if (delay < 100)
                        delay = 100;
                    Bitmap pic = new Bitmap(image);

                    try
                    {
                        writeImage(pic);
                    }
                    catch (PhidgetException ex)
                    {
                        errorBox.addMessage("Error writing image: " + ex.Message);
                        return;
                    }

                    long tm = delay - sw.ElapsedMilliseconds;
                    if (tm > 0)
                        Thread.Sleep((int)tm);

                    fps = 1 / (sw.ElapsedMilliseconds / 1000.0);
                    fpsLbl.BeginInvoke((MethodInvoker)delegate { fpsLbl.Text = fps.ToString("F3") + " fps"; });

                    if (stop_writeAnimatedGifThread)
                        return;
                }
            }
        }

        private void imageBtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            try
            {
                OpenFileDialog open = new OpenFileDialog();
                open.Filter = "Image Files(*.jpg; *.jpeg; *.gif; *.bmp; *.png)|*.jpg; *.jpeg; *.gif; *.bmp; *.png";
                if (open.ShowDialog() == DialogResult.OK)
                {
                    Image originalImg = Image.FromFile(open.FileName);
                    int numberOfFrames = 1;
                    try
                    {
                        numberOfFrames = originalImg.GetFrameCount(FrameDimension.Time);
                    }
                    catch { }
                    if (numberOfFrames > 1)
                    {
                        if (writeAnimatedGifThread != null && writeAnimatedGifThread.IsAlive)
                        {
                            stop_writeAnimatedGifThread = true;
                            writeAnimatedGifThread.Join();
                        }
                        if (writeScreenThread != null && writeScreenThread.IsAlive)
                        {
                            stop_writeScreenThread = true;
                            writeScreenThread.Join();
                        }

                        writeAnimatedGifThread = new Thread(new ThreadStart(writeAnimatedGifThreadFunction));
                        image = originalImg;
                        stop_writeAnimatedGifThread = false;
                        writeAnimatedGifThread.Start();
                    }
                    else
                    {
                        Bitmap pic = new Bitmap(originalImg);
                        writeImageGreyscale(pic);
                    }
                }
            }
            catch (Exception ex)
            {
                errorBox.addMessage("Error loading image: " + ex.Message);
            }
        }
        Thread writeScreenThread;
        Boolean stop_writeScreenThread = false;
        void writeScreenThreadFunction()
        {
            double whitepoint = 0.7;
            while (!stop_writeScreenThread)
            {
                //Capture Screen
                //full pixels
                //Bitmap captureBitmap = new Bitmap(128, 64, PixelFormat.Format32bppArgb);
                // 2x pixels
                //Bitmap captureBitmap = new Bitmap(128*2, 64*2, PixelFormat.Format32bppArgb);
                // 4x pixels
                //Bitmap captureBitmap = new Bitmap(128*4, 64*4, PixelFormat.Format32bppArgb);
                // 8x pixels
                Bitmap captureBitmap = new Bitmap(128*8, 64*8, PixelFormat.Format32bppArgb);
                Rectangle captureRectangle = Screen.AllScreens[0].Bounds;
                Graphics captureGraphics = Graphics.FromImage(captureBitmap);
                // capture middle of screen
                captureGraphics.CopyFromScreen(
                    captureRectangle.Width/2 - captureBitmap.Width/2, 
                    captureRectangle.Height/2 - captureBitmap.Height/2, 
                    0, 0, captureRectangle.Size);

                Stopwatch sw = new Stopwatch();
                sw.Start();

                if (whitepoint >= 0.8)
                    whitepoint = 0.4;
                else
                    whitepoint += 0.2;

                try
                {
                    writeImage(captureBitmap, whitepoint);
                }
                catch (PhidgetException ex)
                {
                    errorBox.addMessage("Error writing image: " + ex.Message);
                    return;
                }

                // XXX - don't delay, just write as fast as possible.
                //long tm = delay - sw.ElapsedMilliseconds;
                //if (tm > 0)
                //    Thread.Sleep((int)tm);

                fps = 1 / (sw.ElapsedMilliseconds / 1000.0);
                fpsLbl.BeginInvoke((MethodInvoker)delegate { fpsLbl.Text = fps.ToString("F3") + " fps"; });

            }
        }

        private void screenBtn_Click(object sender, EventArgs e)
        {
            timer2.Stop();
            try
            {
                if (writeAnimatedGifThread != null && writeAnimatedGifThread.IsAlive)
                {
                    stop_writeAnimatedGifThread = true;
                    writeAnimatedGifThread.Join();
                }
                if (writeScreenThread != null && writeScreenThread.IsAlive)
                {
                    stop_writeScreenThread = true;
                    writeScreenThread.Join();
                }
                writeScreenThread = new Thread(new ThreadStart(writeScreenThreadFunction));
                stop_writeScreenThread = false;
                writeScreenThread.Start();
            }
            catch (Exception ex)
            {
                errorBox.addMessage("Error loading image: " + ex.Message);
            }
        }
    }
}
